;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; SMW Flying Coin and Flying 1-Up (sprites 7E & 7F), by imamelia
;;
;; This is a disassembly of sprites 7E and 7F in SMW, the unused flying red coin and
;; flying yellow 1-Up.
;;
;; Uses first extra bit: YES
;;
;; If the extra bit is clear, the disassembly will act like original sprite 7E, the flying red
;; coin.  If the extra bit is set, the disassembly will act like original sprite 7F, the
;; flying yellow 1-Up.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; defines and tables
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

YAccel:
db $FF,$01

MaxYSpeed:
db $F0,$10

ItemTiles:
db $E8,$24

WingXDispLo:
db $FD,$F5,$07,$07

WingXDispHi:
db $FF,$FF,$00,$00

WingYDisp:
db $FE,$F6,$FE,$F6

WingTilemap:
db $5D,$C6,$5D,$C6

WingTileProp:
db $46,$46,$06,$06

WingTileSize:
db $00,$02,$00,$02

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; init routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

print "INIT ",pc

LDA $7FAB10,x	;
AND #$04	;
LSR #2		;
STA $1510,x	;
BEQ NoChangeCFG	; if the extra bit is set, slightly modify the .cfg settings
LDA #$20		;
STA $166E,x	;
STZ $15F6,x	;
LDA #$82		;
STA $187A,x	;
NoChangeCFG:	;

RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; main routine wrapper
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

print "MAIN ",pc
PHB
PHK
PLB
JSR FlyingItemsMain
PLB
RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; main routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

FlyingItemsMain:

LDA $9D		;
BNE SpritesLocked	; ...I shouldn't have to explain what this does

LDA $13		;
AND #$01	; every other frame...
BNE NoAccel	; make the sprite accelerate vertically

LDA $151C,x	; vertical direction in this case
AND #$01	;
TAY		; index the acceleration tables
LDA $AA,x	;
CLC		;
ADC YAccel,y	; increment or decrement the sprite's Y speed by 1
STA $AA,x	;
CMP MaxYSpeed,y	; if the Y speed has maxed out...
BNE NoAccel	;
INC $151C,x	; switch direction

NoAccel:		;

LDA #$0C		; the sprite's X speed is always the same
STA $B6,x		;

JSL $018022	; update sprite X position without gravity

LDA $AA,x	;
PHA		; preserve the sprite Y speed
;CLC		; now why in the heck is this here??
SEC		;
SBC #$02		; subtract 2 from the Y speed
STA $AA,x	; before updating the Y position

JSL $01801A	; update sprite Y position without gravity

PLA		; I think the purpose for this is to make it fly higher and higher...?
STA $AA,x	; restore the Y speed

JSR SubOffscreenX0	;

INC $1570,x	;

SpritesLocked:	;

; This code, interestingly enough, seems to handle the Yoshi wings.  Yes, the Yoshi wings actually
; are a form of sprite 7E.  The code at $028A03 sets the sprite state to 02 when spawning them so
; that they will act like the wings instead of the coin.

LDA $C2,x	;
CMP #$02		; if the sprite state is 02...
BNE NoFlash	;
LDA $13		;
AND #$03	; every 4th frame...
BNE NoGlitter	;
PHK		;
PER $0006	;
PEA $8020	;
JML $01B14E	; display glitter
NoGlitter:		;
LDA $14		;
LSR		; frame counter 2
AND #$0E	;
EOR $15F6,x	; change the sprite palette
STA $15F6,x	;

NoFlash:

JSR WingGFX	; draw the wings

LDA $C2,x	; check the sprite state
BEQ NotYoshiWings	; if it is nonzero, then draw the wings only
JSR GetDrawInfo	; what the heck is the purpose of this
RTS		;

NotYoshiWings:	;

JSR FlyingItemGFX	; draw the sprite
JSL $01A7DC	; interact with the player
BCC Return00	; if there is no contact, return

LDA $1510,x	; if the extra bit is clear...
BNE Give1Up	; then the sprite is a red coin and not a 1-Up

PHK		;
PER $0006	;
PEA $8020	;
JML $01C4F0	; display glitter (different kind)

LDA #$05		; give 5 coins
JSL $05B329	;
LDA #$03		; give 800 points
JSL $02ACE5	;
BRA EraseSprite0	; erase the sprite

Give1Up:

LDA #$08		; the original JSR'd to $01C5FE here and added $1594,x to the point value,
JSL $02ACE5	; which was never changed in the first place in this sprite

EraseSprite0:

STZ $14C8,x	;

Return00:		;
RTS		;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; graphics routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

FlyingItemGFX:

JSR GetDrawInfo	;

STZ $0A		;
LDA $140F	;
BNE NoSpecialMode	;
LDA $0D9B	;
CMP #$C1	;
BEQ NoSpecialMode	;
BIT $0D9B	;
BVC NoSpecialMode	;
LDA #$D8	; if the player is in a special boss level...
STA $15EA,x	; hardcode the sprite's OAM index
TAY		;

NoSpecialMode:	;

LDA $00		;
STA $0300,y	;
LDA $01		;
DEC		; Y displacement of -1
STA $0301,y	;

LDA #$40		; I don't think this was supposed to be here, but...
ORA $64		; yes, the tile is always X-flipped because the sprite direction is always 0
ORA $15F6,x	;
STA $0303,y	;

LDA $1510,x	; extra bit to set the tile number
TAX		;
LDA ItemTiles,x	;
STA $0302,y	;

LDX $15E9	;
LDY #$02		;
LDA #$00		;
JSL $01B7B3	;
RTS		;

WingGFX:

LDA $E4,x	;
PHA		;
LDA $14E0,x	;
PHA		;
LDA $15EA,x	;
PHA		;
CLC		;
ADC #$04	; shift the OAM index
STA $15EA,x	;
LDA $157C,x	;
PHA		;
STZ $157C,x	; set the sprite direction to 0

LDA $1570,x	;
LSR #3		;
AND #$01	;
TAY		;

JSR WingGFXEntry0	;

LDA $E4,x	;
CLC		;
ADC #$04	;
STA $E4,x		;
LDA $14E0,x	;
ADC #$00	;
STA $14E0,x	;

LDA $15EA,x	;
CLC		;
ADC #$04	;
STA $15EA,x	;

INC $157C,x	;

JSR WingGFXEntry1	;

PLA		;
STA $157C,x	;
PLA		;
STA $15EA,x	;
PLA		;
STA $14E0,x	;
PLA		;
STA $E4,x		;

RTS		;

WingGFXEntry0:	;

STY $02		;

WingGFXEntry1:	;

LDA $186C,x	;
BNE Return01	;

LDA $E4,x	;
STA $00		;
LDA $14E0,x	;
STA $04		;
LDA $D8,x	;
STA $01		;
LDY $15EA,x	;
PHX		;
LDA $157C,x	;
ASL		;
ADC $02		;
TAX		;

LDA $00		;
CLC		;
ADC WingXDispLo,x	;
STA $00		;
LDA $04		;
ADC WingXDispHi,x	;
PHA		;
LDA $00		;
SEC		;
SBC $1A		;
STA $0300,y	;
PLA		;
SBC $1B		;
BNE NoWingTiles	;

LDA $01		;
SEC		;
SBC $1C		;
CLC		;
ADC WingYDisp,x	;
STA $0301,y	;

LDA WingTilemap,x	;
STA $0302,y	;

LDA $64		;
ORA WingTileProp,x	;
STA $0303,y	;

TYA		;
LSR #2		;
TAY		;
LDA WingTileSize,x	;
STA $0460,y	;

NoWingTiles:

PLX

Return01:
RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; miscellaneous standard subroutines
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Table1:              db $0C,$1C
Table2:              db $01,$02
Table3:              db $40,$B0
Table6:              db $01,$FF
Table4:              db $30,$C0,$A0,$C0,$A0,$F0,$60,$90,$30,$C0,$A0,$80,$A0,$40,$60,$B0
Table5:              db $01,$FF,$01,$FF,$01,$FF,$01,$FF,$01,$FF,$01,$FF,$01,$00,$01,$FF

SubOffscreenX0:
LDA #$00
;BRA SubOffscreenMain
;SubOffscreenX1:
;LDA #$02
;BRA SubOffscreenMain
;SubOffscreenX2:
;LDA #$04
;BRA SubOffscreenMain
;SubOffscreenX3:
;LDA #$06
;BRA SubOffscreenMain
;SubOffscreenX4:
;LDA #$08
;BRA SubOffscreenMain
;SubOffscreenX5:
;LDA #$0A
;BRA SubOffscreenMain
;SubOffscreenX6:
;LDA #$0C
;BRA SubOffscreenMain
;SubOffscreenX7:
;LDA #$0E

SubOffscreenMain:

STA $03

JSR SubIsOffscreen
BEQ Return2

LDA $5B
LSR
BCS VerticalLevel
LDA $D8,x
CLC
ADC #$50
LDA $14D4,x
ADC #$00
CMP #$02
BPL EraseSprite
LDA $167A,x
AND #$04
BNE Return2
LDA $13
AND #$01
ORA $03
STA $01
TAY
LDA $1A
CLC
ADC Table4,y
ROL $00
CMP $E4,x
PHP
LDA $1B
LSR $00
ADC Table5,y
PLP
SBC $14E0,x
STA $00
LSR $01
BCC Label20
EOR #$80
STA $00
Label20:
LDA $00
BPL Return2

EraseSprite:
LDA $14C8,x
CMP #$08
BCC KillSprite
LDY $161A,x
CPY #$FF
BEQ KillSprite
LDA #$00
STA $1938,y
KillSprite:
STZ $14C8,x
Return2:
RTS

VerticalLevel:

LDA $167A,x
AND #$04
BNE Return2
LDA $13
LSR
BCS Return2
AND #$01
STA $01
TAY
LDA $1C
CLC
ADC Table3,y
ROL $00
CMP $D8,x
PHP
LDA $1D
LSR $00
ADC Table6,y
PLP
SBC $14D4,x
STA $00
LDY $02
BEQ Label22
EOR #$80
STA $00
Label22:
LDA $00
BPL Return2
BMI EraseSprite

SubIsOffscreen:
LDA $15A0,x
ORA $186C,x
RTS

GetDrawInfo:

STZ $186C,x
STZ $15A0,x
LDA $E4,x
CMP $1A
LDA $14E0,x
SBC $1B
BEQ OnscreenX
INC $15A0,x
OnscreenX:
LDA $14E0,x
XBA
LDA $E4,x
REP #$20
SEC
SBC $1A
CLC
ADC.w #$0040
CMP #$0180
SEP #$20
ROL A
AND #$01
STA $15C4,x
BNE Invalid

LDY #$00
LDA $1662,x
AND #$20
BEQ OnscreenLoop
INY
OnscreenLoop:
LDA $D8,x
CLC
ADC Table1,y
PHP
CMP $1C
ROL $00
PLP
LDA $14D4,x
ADC #$00
LSR $00
SBC $1D
BEQ OnscreenY
LDA $186C,x
ORA Table2,y
STA $186C,x
OnscreenY:
DEY
BPL OnscreenLoop
LDY $15EA,x
LDA $E4,x
SEC
SBC $1A
STA $00
LDA $D8,x
SEC
SBC $1C
STA $01
RTS

Invalid:
PLA
PLA
RTS











